#!/bin/sh

trap auxilium_close INT QUIT TERM

if [ "$(basename $0)" = "auxilium" ]; then
    echo "You can't use this script directly"
    echo "You can view an example by call /usr/bin/auxilium-test"
    exit
fi

_to_lower(){
    echo "$1" |tr -s '[:upper:]' '[:lower:]'
}

_to_upper(){
    echo "$1" |tr -s '[:lower:]' '[:upper:]'
}

auxilium_init() {
    # TODO: Add manage for subcommand
    # Param :
    # App NAME : Mandatory
    # -s: Name of subcommand
    # -m: Help description for Application
    while [ $# -gt 0 ]; do
        local _key="$1"
        case $_key in
            -s)
                local subname=$2
                shift 2
                ;;
            -m)
                readonly _AXM_APP_DESCR=$2
                shift 2
                ;;
            *)
                readonly _AXM_APP_NAME=$(_to_lower $_key)
                shift 1
                ;;
        esac
    done
    if [ -z "$subname" ]; then
        readonly __AXM_FILE_PREFIX="${_AXM_APP_NAME}"
    else
        readonly __AXM_FILE_PREFIX="${_AXM_APP_NAME}_${subname}"
    fi
    readonly _AXM_ARGS_FILE=$(mktemp -t ${__AXM_FILE_PREFIX}-auxilium_args.XXXXXX)
    readonly _AXM_ARGS_VALUE=$(mktemp -t ${__AXM_FILE_PREFIX}-auxilium_value.XXXXXX)
    readonly _AXM_LIST_ARGS=$(mktemp -t ${__AXM_FILE_PREFIX}-auxilium_list_args.XXXXXX)
    readonly _AXM_LIST_OPT=$(mktemp -t ${__AXM_FILE_PREFIX}-auxilium_list_opt.XXXXXX)
    auxilium_add help -s h -t opt -m 'This help message' -c auxilium_usage
    auxilium_add 'sh_completion' -t str -l "bash|zsh" -m 'Generate shell completion script' -c _auxilium_compl
}

auxilium_add() {
    # TODO: Add manage for subcommand
    # Param:
    # Opt NAME: Mandatory
    # -t type of argument (str by default|opt|positional|subcommand)
    # -s Short option
    # -d Default value
    # -a All remaining arguments, must be last
    # -l List of choices separate by |
    # -c Callback fonction (Arg in parameter)
    # -m Help Message
    local type='str'
    local all=0
    while [ $# -gt 0 ]; do
        local _key="$1"
        case $_key in
            -s)
                local short=$2
                shift 2
                ;;
            -t)
                type=$2
                shift 2
                ;;
            -d)
                local default=$2
                shift 2
                ;;
            -a)
                all=1
                shift 1
                ;;
            -l)
                local choice=$2
                shift 2
                ;;
            -c)
                local callback=$2
                shift 2
                ;;
            -m)
                local message=$2
                shift 2
                ;;
            *)
                local name=$(_to_lower $_key)
                shift 1
                ;;
        esac
    done
    #NAME^TYPE^SHORT^DEFAULT^ALL^CHOICE^CALLBACK^MESSAGE
    printf "%s^%s^%s^%s^%s^%s^%s^%s\n" \
        "$name" \
        "$type" \
        "$short" \
        "$default" \
        "$all" \
        "$choice" \
        "$callback"  \
        "$message" >> "${_AXM_ARGS_FILE}"
}

auxilium_usage(){
    # TODO: Add manage for subcommand
    tabs 4,20,35,45,60
    # Print usage
    local output_args=''
    local output_pos=''
    local usage_args=''
    local usage_pos=''

    printf "Description:\n"
    printf "\t%s\n" "${_AXM_APP_DESCR}"
    while IFS=^ read -r name type short default all choice callback message
    do
        local def=''
        local noopt=''
        local name_upper=$(_to_upper ${name})
        local name_lower=$(_to_lower ${name})
        local opt="--${name_lower}"
        if [ ! -z "${short}" ]; then
            opt="-${short}|${opt}"
        fi
        if [ "${type}" != "opt" ];then
            if [ ${all} -eq 1 ]; then
                noopt="${name_upper}..."
            else
                noopt="${name_upper}"
           fi
        fi
        if [ -n "${default}" ]; then
            def="$(printf "Default: %s" "${default}")"
        elif [ -n "${choice}" ]; then
            def="$(printf "Choice: %s" "${choice}")"
        fi
        local msg=$(printf "${message}" |awk -vRS="\n" -vORS="\n\t\t\t" '1' | sed '$d' )
        if [ "${type}" = "positional" ];then
            output_pos="${output_pos}\n$(printf '\t%s\n\t\t\t%s\n' "$noopt" "$msg")"
            usage_pos="${usage_pos} $(printf '<%s>' "$noopt")"
        else
            output_args="${output_args}\n$(printf '\t%s %s\t%s\n\t\t\t%s' "$opt" "$noopt" "$def" "$msg")"
            usage_args="${usage_args} $(printf '[%s %s]' "$opt" "${noopt}")"
        fi
    done < ${_AXM_ARGS_FILE}
    printf "\nUsage: "
    printf "${_AXM_APP_NAME}${usage_args}${usage_pos}\n"
    printf "\nOptionals Arguments:"
    printf "${output_args}\n"
    printf "\nPositional Argument:"
    printf "${output_pos}\n\n"
    tabs -8
    exit 0
}

_auxilium_compl(){
    # TODO: Add manage for subcommand
    local opts=''
    local args=''
    local map=''
    while IFS=^ read -r name type short default all choice callback message
    do
        [ "$type" == "positional" ] && continue
        local name_lower=$(_to_lower ${name})
        local s=''
        local c=''
        if [ "$2" == "bash" ]; then
            if [ ! -z "${short}" ]; then
                s=" -${short}"
                map="${map} [-${short}]=${name_lower}"
            fi
            opts="${opts} [${name_lower}]=\'--${name_lower}${s}\'"
            map="${map} [--${name_lower}]=${name_lower}"
            if [ "${type}" == "str" ];then
                c=$(echo "$choice"|sed s/'|'/' '/g)
                args="${args} [${name_lower}]=\'${choice}\'"
            fi
        elif [ "$2" == "zsh" ]; then
            if [ ! -z "${short}" ]; then
                s="\'(--${name_lower} -${short})\'{--${name_lower},-${short}}"
            else
                s="--${name_lower}"
            fi
            if [ "${type}" == "str" ];then
                local ch=$(echo "$choice"|sed s/'|'/' '/g)
                c=":choice:(${ch})"
            else
                c=''
            fi
            local m=$(printf "${message}" | sed "s/\//\\\\\//g" |awk -vRS="\n" -vORS=" " '1')
            opts="${opts}    ${s}\'[${m}]\'${c} \\\\\n"
        fi
    done < ${_AXM_ARGS_FILE}
    local template="/usr/share/auxilium/${2}-completion.template"
    if [ -f "/etc/auxilium/${2}-completion.template" ]; then
        template="/etc/auxilium/${2}-completion.template"
    fi
    cat $template \
        | sed -e "s/{scriptname}/${_AXM_APP_NAME}/g" \
        | sed -e "s/{opts}/${opts} /g" \
        | sed -e "s/{args}/${args} /g" \
        | sed -e "s/{map}/${map} /g"
    exit 0
}

auxilium_parse(){
    # TODO: Add manage for subcommand
    # Param: Args of the App
    local list_args=''
    local pos=1
    cat ${_AXM_ARGS_FILE}| grep -v "\^positional\^" > "$_AXM_LIST_ARGS"
    # Default Values
    while IFS=^ read -r name type short default all choice callback message
    do
        if [ ! -z "${default}" ]; then
            echo "AUXILIUM_ARG_${name}='${default}'" >> "$_AXM_ARGS_VALUE"
        fi
    done < $_AXM_LIST_ARGS
    while [ $# -gt 0 ] ;
    do
        local arg="${1}"
        local axm_find=0
        while IFS=^ read -r name type short default all choice callback message
        do
            local name_lower="$(_to_lower ${name})"
            local opt="^--${name_lower}$"
            if [ ! -z "${short}" ]; then
                opt="^-${short}$\|${opt}"
            fi
            if  expr match $arg $opt > /dev/null ; then
                axm_find=1
                if [ "${type}" = "opt" ];then
                    echo "readonly AUXILIUM_ARG_${name}='1'" >> "$_AXM_ARGS_VALUE"
                    shift 1
                    if [ ! -z "${callback}" ]; then
                        ${callback} ${name} 1
                    fi
                else
                    shift 1
                    if [ ! -z "${choice}" ]; then
                        if (echo "|${choice}|" | grep -v "|${1}|"); then
                            echo "ERROR: ${1} is not in list of choices ${choice} for ${arg}"
                            exit 1
                        fi
                    fi
                     if [ ${all} -eq 1 ]; then
                        list_args=$@
                        shift $#
                    else
                        list_args=${1}
                        shift 1
                    fi
                    echo "readonly AUXILIUM_ARG_${name}='${list_args}'" >> "$_AXM_ARGS_VALUE"
                    if [ ! -z "${callback}" ]; then
                        ${callback} ${name} ${list_args} 2>/dev/null
                    fi
                fi
                break
            fi
        done < $_AXM_LIST_ARGS
        if [ $axm_find -eq 0 ]; then
            cat ${_AXM_ARGS_FILE}| grep "\^positional\^"| head -${pos}|tail -1 > "$_AXM_LIST_OPT"
            IFS=^ read -r name type short default all choice callback message < "$_AXM_LIST_OPT"
            if [ ${all} -eq 1 ]; then
                list_args=$@
                shift $#
            else
                list_args=$1
                shift 1
                
            fi
            echo "readonly AUXILIUM_ARG_${name}='${list_args}'" >> "$_AXM_ARGS_VALUE"
            pos=$(($pos+1))
            if [ ! -z "${callback}" ]; then
                ${callback} ${name} ${list_args} 2>/dev/null
            fi
        fi
    done

    # Set de new variable by source de new temporary file
    . "${_AXM_ARGS_VALUE}"
}

auxilium_close(){
    # Delete temporaries files
    rm -f "${_AXM_ARGS_FILE}" 2>/dev/null
    rm -f "${_AXM_ARGS_VALUE}" 2>/dev/null
    rm -f "${_AXM_LIST_ARGS}" 2>/dev/null
    rm -f "${_AXM_LIST_OPT}" 2>/dev/null
    # TODO: Add rm subcommand file
}
